@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -34,6 +29,7 @@
 #include <cplusplus/Lexer.h>
 
 #include <utils/qtcassert.h>
+#include <utils/qtcfallthrough.h>
 
 #include <QDebug>
 #include <QMetaEnum>
@@ -45,18 +41,6 @@ using namespace CppTools;
 using namespace TextEditor;
 using namespace CppTools::Internal;
 
-CodeFormatter::BlockData::BlockData()
-    : m_blockRevision(-1)
-{
-}
-
-CodeFormatter::CodeFormatter()
-    : m_indentDepth(0)
-    , m_paddingDepth(0)
-    , m_tabSize(4)
-{
-}
-
 CodeFormatter::~CodeFormatter()
 {
 }
@@ -688,11 +672,6 @@ CodeFormatter::State CodeFormatter::state(int belowTop) const
         return State();
 }
 
-const QVector<CodeFormatter::State> &CodeFormatter::newStatesThisLine() const
-{
-    return m_newStates;
-}
-
 int CodeFormatter::tokenIndex() const
 {
     return m_tokenIndex;
@@ -703,7 +682,7 @@ int CodeFormatter::tokenCount() const
     return m_tokens.size();
 }
 
-const CPlusPlus::Token &CodeFormatter::currentToken() const
+const Token &CodeFormatter::currentToken() const
 {
     return m_currentToken;
 }
@@ -870,7 +849,7 @@ bool CodeFormatter::tryDeclaration()
                 return true;
             }
         }
-        // fallthrough
+        Q_FALLTHROUGH();
     case T_CHAR:
     case T_CHAR16_T:
     case T_CHAR32_T:
@@ -1076,6 +1055,7 @@ int CodeFormatter::tokenizeBlock(const QTextBlock &block, bool *endedJoined)
     features.qtEnabled = true;
     features.qtMocRunEnabled = true;
     features.qtKeywordsEnabled = true;
+    features.cxxEnabled = true;
     features.objCEnabled = true;
 
     SimpleLexer tokenize;
@@ -1111,7 +1091,7 @@ void CodeFormatter::dump() const
 
 namespace CppTools {
 namespace Internal {
-    class CppCodeFormatterData: public TextEditor::CodeFormatterData
+    class CppCodeFormatterData: public CodeFormatterData
     {
     public:
         CodeFormatter::BlockData m_data;
@@ -1123,7 +1103,7 @@ QtStyleCodeFormatter::QtStyleCodeFormatter()
 {
 }
 
-QtStyleCodeFormatter::QtStyleCodeFormatter(const TextEditor::TabSettings &tabSettings,
+QtStyleCodeFormatter::QtStyleCodeFormatter(const TabSettings &tabSettings,
                                            const CppCodeStyleSettings &settings)
     : m_tabSettings(tabSettings)
     , m_styleSettings(settings)
@@ -1131,7 +1111,7 @@ QtStyleCodeFormatter::QtStyleCodeFormatter(const TextEditor::TabSettings &tabSet
     setTabSize(tabSettings.m_tabSize);
 }
 
-void QtStyleCodeFormatter::setTabSettings(const TextEditor::TabSettings &tabSettings)
+void QtStyleCodeFormatter::setTabSettings(const TabSettings &tabSettings)
 {
     m_tabSettings = tabSettings;
     setTabSize(tabSettings.m_tabSize);
@@ -1252,7 +1232,7 @@ void QtStyleCodeFormatter::onEnter(int newState, int *indentDepth, int *savedInd
     case assign_open:
         if (parentState.type == assign_open_or_initializer)
             break;
-        // fallthrough
+        Q_FALLTHROUGH();
     case assign_open_or_initializer:
         if (!lastToken && m_styleSettings.alignAssignments)
             *paddingDepth = nextTokenPosition-*indentDepth;
