@@ -23,8 +23,7 @@
 **
 ****************************************************************************/
 
-#ifndef DEBUGGERCORE_H
-#define DEBUGGERCORE_H
+#pragma once
 
 #include "debuggerconstants.h"
 
@@ -33,11 +32,13 @@
 #include <QObject>
 #include <QSharedPointer>
 
+#include <functional>
+
 QT_BEGIN_NAMESPACE
-class QIcon;
 class QMessageBox;
 class QWidget;
-class QTreeView;
+class QMenu;
+class QAction;
 QT_END_NAMESPACE
 
 namespace CPlusPlus { class Snapshot; }
@@ -45,6 +46,9 @@ namespace CPlusPlus { class Snapshot; }
 namespace Utils { class SavedAction; }
 
 namespace Debugger {
+
+class DebuggerRunTool;
+
 namespace Internal {
 
 class BreakHandler;
@@ -52,6 +56,7 @@ class DebuggerEngine;
 class Symbol;
 class Section;
 class GlobalDebuggerOptions;
+class WatchTreeView;
 
 enum TestCases
 {
@@ -60,9 +65,8 @@ enum TestCases
 };
 
 // Some convenience.
-void updateState(DebuggerEngine *engine);
-void updateWatchersWindow(bool showWatch, bool showReturn);
-QIcon locationMarkIcon();
+void updateState(DebuggerRunTool *runTool);
+void updateLocalsWindow(bool showReturn);
 const CPlusPlus::Snapshot &cppCodeModelSnapshot();
 bool hasSnapshots();
 void openTextEditor(const QString &titlePattern, const QString &contents);
@@ -71,13 +75,14 @@ void openTextEditor(const QString &titlePattern, const QString &contents);
 void showMessage(const QString &msg, int channel, int timeout = -1);
 
 bool isReverseDebugging();
-void runControlStarted(DebuggerEngine *engine);
-void runControlFinished(DebuggerEngine *engine);
-void displayDebugger(DebuggerEngine *engine, bool updateEngine);
+void runControlStarted(DebuggerRunTool *runTool);
+void runControlFinished(DebuggerRunTool *runTool);
+void displayDebugger(DebuggerRunTool *runTool, bool updateEngine);
 void synchronizeBreakpoints();
 
 void saveModeToRestore();
 QWidget *mainWindow();
+void raiseWatchersWindow();
 bool isRegistersWindowVisible();
 bool isModulesWindowVisible();
 void showModuleSymbols(const QString &moduleName, const QVector<Internal::Symbol> &symbols);
@@ -88,11 +93,11 @@ void setThreadBoxContents(const QStringList &list, int index);
 
 QSharedPointer<Internal::GlobalDebuggerOptions> globalDebuggerOptions();
 
-QTreeView *inspectorView();
+WatchTreeView *inspectorView();
 QVariant sessionValue(const QByteArray &name);
 void setSessionValue(const QByteArray &name, const QVariant &value);
-QVariant configValue(const QByteArray &name);
-void setConfigValue(const QByteArray &name, const QVariant &value);
+QVariant configValue(const QString &name);
+void setConfigValue(const QString &name, const QVariant &value);
 
 bool isTestRun();
 
@@ -109,7 +114,17 @@ QMessageBox *showMessageBox(int icon, const QString &title,
 
 bool isReverseDebuggingEnabled();
 
+QAction *addAction(QMenu *menu, const QString &display, bool on,
+                   const std::function<void()> &onTriggered = {});
+QAction *addAction(QMenu *menu, const QString &d1, const QString &d2, bool on,
+                   const std::function<void()> &onTriggered);
+QAction *addCheckableAction(QMenu *menu, const QString &display, bool on, bool checked,
+                            const std::function<void()> &onTriggered);
+
+// Qt's various build paths for unpatched versions
+QStringList qtBuildPaths();
+
+void addDebugInfoTask(unsigned id, const QString &cmd);
+
 } // namespace Internal
 } // namespace Debugger
-
-#endif // DEBUGGERPLUGIN_H
