@@ -23,14 +23,17 @@
 **
 ****************************************************************************/
 
-#ifndef DIFFUTILS_H
-#define DIFFUTILS_H
+#pragma once
 
 #include "diffeditor_global.h"
 
 #include <QString>
 #include <QMap>
 
+QT_BEGIN_NAMESPACE
+class QFutureInterfaceBase;
+QT_END_NAMESPACE
+
 namespace TextEditor { class FontSettings; }
 
 namespace DiffEditor {
@@ -39,12 +42,18 @@ class Diff;
 
 class DIFFEDITOR_EXPORT DiffFileInfo {
 public:
+    enum PatchBehaviour {
+        PatchFile,
+        PatchEditor
+    };
+
     DiffFileInfo() {}
     DiffFileInfo(const QString &file) : fileName(file) {}
     DiffFileInfo(const QString &file, const QString &type)
         : fileName(file), typeInfo(type) {}
     QString fileName;
     QString typeInfo;
+    PatchBehaviour patchBehaviour = PatchFile;
 };
 
 class DIFFEDITOR_EXPORT TextLineData {
@@ -54,10 +63,10 @@ public:
         Separator,
         Invalid
     };
-    TextLineData() : textLineType(Invalid) {}
+    TextLineData() {}
     TextLineData(const QString &txt) : textLineType(TextLine), text(txt) {}
     TextLineData(TextLineType t) : textLineType(t) {}
-    TextLineType textLineType;
+    TextLineType textLineType = Invalid;
     QString text;
     /*
      * <start position, end position>
@@ -70,24 +79,23 @@ public:
 
 class DIFFEDITOR_EXPORT RowData {
 public:
-    RowData() : equal(false) {}
+    RowData() {}
     RowData(const TextLineData &l)
         : leftLine(l), rightLine(l), equal(true) {}
     RowData(const TextLineData &l, const TextLineData &r)
-        : leftLine(l), rightLine(r), equal(false) {}
+        : leftLine(l), rightLine(r) {}
     TextLineData leftLine;
     TextLineData rightLine;
-    bool equal;
+    bool equal = false;
 };
 
 class DIFFEDITOR_EXPORT ChunkData {
 public:
-    ChunkData() : contextChunk(false),
-        leftStartingLineNumber(0), rightStartingLineNumber(0) {}
+    ChunkData() {}
     QList<RowData> rows;
-    bool contextChunk;
-    int leftStartingLineNumber;
-    int rightStartingLineNumber;
+    bool contextChunk = false;
+    int leftStartingLineNumber = 0;
+    int rightStartingLineNumber = 0;
     QString contextInfo;
 };
 
@@ -95,29 +103,22 @@ class DIFFEDITOR_EXPORT FileData {
 public:
     enum FileOperation {
         ChangeFile,
+        ChangeMode,
         NewFile,
         DeleteFile,
         CopyFile,
         RenameFile
     };
 
-    FileData()
-        : fileOperation(ChangeFile),
-          binaryFiles(false),
-          lastChunkAtTheEndOfFile(false),
-          contextChunksIncluded(false) {}
-    FileData(const ChunkData &chunkData)
-        : fileOperation(ChangeFile),
-          binaryFiles(false),
-          lastChunkAtTheEndOfFile(false),
-          contextChunksIncluded(false) { chunks.append(chunkData); }
+    FileData() {}
+    FileData(const ChunkData &chunkData) { chunks.append(chunkData); }
     QList<ChunkData> chunks;
     DiffFileInfo leftFileInfo;
     DiffFileInfo rightFileInfo;
-    FileOperation fileOperation;
-    bool binaryFiles;
-    bool lastChunkAtTheEndOfFile;
-    bool contextChunksIncluded;
+    FileOperation fileOperation = ChangeFile;
+    bool binaryFiles = false;
+    bool lastChunkAtTheEndOfFile = false;
+    bool contextChunksIncluded = false;
 };
 
 class DIFFEDITOR_EXPORT DiffUtils {
@@ -145,9 +146,8 @@ public:
     static QString makePatch(const QList<FileData> &fileDataList,
                              unsigned formatFlags = 0);
     static QList<FileData> readPatch(const QString &patch,
-                                     bool *ok = 0);
+                                     bool *ok = 0,
+                                     QFutureInterfaceBase *jobController = nullptr);
 };
 
 } // namespace DiffEditor
-
-#endif // DIFFUTILS_H
