@@ -39,9 +39,7 @@
 #include <functional>
 
 QT_BEGIN_NAMESPACE
-class QFile;
 class QAction;
-class QFileInfo;
 QT_END_NAMESPACE
 
 namespace Core {
@@ -61,7 +59,6 @@ namespace Internal {
 
 class GitVersionControl;
 class GitClient;
-class GitSubmitEditor;
 class CommitData;
 class StashDialog;
 class BranchDialog;
@@ -87,9 +84,9 @@ public:
 
     Gerrit::Internal::GerritPlugin *gerritPlugin() const;
     bool isCommitEditorOpen() const;
-
-public slots:
-    void startCommit();
+    static QString msgRepositoryLabel(const QString &repository);
+    static QString invalidBranchAndRemoteNamePattern();
+    void startCommit(CommitType commitType = SimpleCommit);
     void updateBranches(const QString &repository);
 
 protected:
@@ -115,6 +112,7 @@ private:
     void logRepository();
     void undoFileChanges(bool revertStaging);
     void resetRepository();
+    void recoverDeletedFiles();
     void startRebase();
     void startChangeRelatedAction(const Core::Id &id);
     void stageFile();
@@ -128,8 +126,6 @@ private:
     void applyCurrentFilePatch();
     void promptApplyPatch();
 
-    void startAmendCommit();
-    void startFixupCommit();
     void stash(bool unstagedOnly = false);
     void stashUnstaged();
     void stashSnapshot();
@@ -160,11 +156,6 @@ private:
                               Core::Id id, const Core::Context &context, bool addToLocator,
                               const std::function<void()> &callback,
                               const QKeySequence &keys = QKeySequence());
-    QAction *createFileAction(Core::ActionContainer *ac,
-                              const QString &defaultText, const QString &parameterText,
-                              Core::Id id, const Core::Context &context, bool addToLocator,
-                              void (GitPlugin::*func)(),
-                              const QKeySequence &keys = QKeySequence());
 
     QAction *createProjectAction(Core::ActionContainer *ac,
                                  const QString &defaultText, const QString &parameterText,
@@ -188,7 +179,6 @@ private:
     void cleanCommitMessageFile();
     void cleanRepository(const QString &directory);
     void applyPatch(const QString &workingDirectory, QString file = QString());
-    void startCommit(CommitType commitType);
     void updateVersionWarning();
 
     Core::CommandLocator *m_commandLocator = nullptr;
@@ -205,6 +195,7 @@ private:
     QAction *m_abortRebaseAction = nullptr;
     QAction *m_abortCherryPickAction = nullptr;
     QAction *m_abortRevertAction = nullptr;
+    QAction *m_skipRebaseAction = nullptr;
     QAction *m_continueRebaseAction = nullptr;
     QAction *m_continueCherryPickAction = nullptr;
     QAction *m_continueRevertAction = nullptr;
