@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -32,10 +27,12 @@
 #include "ui_desktopdeviceconfigurationwidget.h"
 #include <projectexplorer/projectexplorerconstants.h>
 
-#include <coreplugin/coreconstants.h>
+#include <utils/utilsicons.h>
 #include <utils/portlist.h>
 #include <utils/qtcassert.h>
 
+#include <QRegExpValidator>
+
 using namespace ProjectExplorer::Constants;
 
 namespace ProjectExplorer {
@@ -46,8 +43,8 @@ DesktopDeviceConfigurationWidget::DesktopDeviceConfigurationWidget(const IDevice
     m_ui(new Ui::DesktopDeviceConfigurationWidget)
 {
     m_ui->setupUi(this);
-    connect(m_ui->freePortsLineEdit, SIGNAL(textChanged(QString)),
-            SLOT(updateFreePorts()));
+    connect(m_ui->freePortsLineEdit, &QLineEdit::textChanged,
+            this, &DesktopDeviceConfigurationWidget::updateFreePorts);
 
     initGui();
 }
@@ -74,8 +71,7 @@ void DesktopDeviceConfigurationWidget::initGui()
     m_ui->machineTypeValueLabel->setText(tr("Physical Device"));
     m_ui->freePortsLineEdit->setPlaceholderText(
                 QString::fromLatin1("eg: %1-%2").arg(DESKTOP_PORT_START).arg(DESKTOP_PORT_END));
-    m_ui->portsWarningLabel->setPixmap(
-                QPixmap(QLatin1String(Core::Constants::ICON_WARNING)));
+    m_ui->portsWarningLabel->setPixmap(Utils::Icons::WARNING.pixmap());
     m_ui->portsWarningLabel->setToolTip(QLatin1String("<font color=\"red\">")
                                         + tr("You will need at least one port for QML debugging.")
                                         + QLatin1String("</font>"));
