@@ -29,8 +29,8 @@
 #include "qbsbuildstep.h"
 #include "qbscleanstep.h"
 #include "qbsdeployconfigurationfactory.h"
-#include "qbsinfopage.h"
 #include "qbsinstallstep.h"
+#include "qbskitinformation.h"
 #include "qbsnodes.h"
 #include "qbsprofilessettingspage.h"
 #include "qbsproject.h"
@@ -44,17 +44,22 @@
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/editormanager/ieditor.h>
 #include <coreplugin/featureprovider.h>
+#include <coreplugin/helpmanager.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/idocument.h>
 #include <coreplugin/fileiconprovider.h>
+
 #include <projectexplorer/buildmanager.h>
 #include <projectexplorer/project.h>
 #include <projectexplorer/projectexplorer.h>
-#include <projectexplorer/projecttree.h>
 #include <projectexplorer/projectexplorerconstants.h>
+#include <projectexplorer/projectmanager.h>
+#include <projectexplorer/projecttree.h>
 #include <projectexplorer/session.h>
 #include <projectexplorer/target.h>
+
 #include <qtsupport/qtsupportconstants.h>
+#include <qmljstools/qmljstoolsconstants.h>
 
 #include <utils/fileutils.h>
 #include <utils/qtcassert.h>
@@ -70,23 +75,15 @@ namespace Internal {
 static Node *currentEditorNode()
 {
     Core::IDocument *doc = Core::EditorManager::currentDocument();
-    return doc ? SessionManager::nodeForFile(doc->filePath()) : 0;
+    return doc ? ProjectTree::nodeForFile(doc->filePath()) : nullptr;
 }
 
 static QbsProject *currentEditorProject()
 {
     Core::IDocument *doc = Core::EditorManager::currentDocument();
-    return doc ? qobject_cast<QbsProject *>(SessionManager::projectForFile(doc->filePath())) : 0;
+    return doc ? qobject_cast<QbsProject *>(SessionManager::projectForFile(doc->filePath())) : nullptr;
 }
 
-QbsProjectManagerPlugin::QbsProjectManagerPlugin() :
-    m_selectedProject(0),
-    m_selectedNode(0),
-    m_currentProject(0),
-    m_editorProject(0),
-    m_editorNode(0)
-{ }
-
 bool QbsProjectManagerPlugin::initialize(const QStringList &arguments, QString *errorMessage)
 {
     Q_UNUSED(arguments);
@@ -95,6 +92,10 @@ bool QbsProjectManagerPlugin::initialize(const QStringList &arguments, QString *
     const Core::Context projectContext(::QbsProjectManager::Constants::PROJECT_ID);
 
     Core::FileIconProvider::registerIconOverlayForSuffix(ProjectExplorer::Constants::FILEOVERLAY_QT, "qbs");
+    Core::HelpManager::registerDocumentation({Core::ICore::documentationPath() + "/qbs.qch"});
+
+    ProjectManager::registerProjectType<QbsProject>(QmlJSTools::Constants::QBS_MIMETYPE);
+    KitManager::registerKitInformation(new QbsKitInformation);
 
     //create and register objects
     addAutoReleasedObject(new QbsManager);
@@ -105,7 +106,6 @@ bool QbsProjectManagerPlugin::initialize(const QStringList &arguments, QString *
     addAutoReleasedObject(new QbsDeployConfigurationFactory);
     addAutoReleasedObject(new QbsRunConfigurationFactory);
     addAutoReleasedObject(new QbsProfilesSettingsPage);
-    addAutoReleasedObject(new QbsInfoPage);
 
     //menus
     // Build Menu:
@@ -127,19 +127,22 @@ bool QbsProjectManagerPlugin::initialize(const QStringList &arguments, QString *
     command = Core::ActionManager::registerAction(m_reparseQbs, Constants::ACTION_REPARSE_QBS, projectContext);
     command->setAttribute(Core::Command::CA_Hide);
     mbuild->addAction(command, ProjectExplorer::Constants::G_BUILD_BUILD);
-    connect(m_reparseQbs, SIGNAL(triggered()), this, SLOT(reparseCurrentProject()));
+    connect(m_reparseQbs, &QAction::triggered,
+            this, &QbsProjectManagerPlugin::reparseCurrentProject);
 
     m_reparseQbsCtx = new QAction(tr("Reparse Qbs"), this);
     command = Core::ActionManager::registerAction(m_reparseQbsCtx, Constants::ACTION_REPARSE_QBS_CONTEXT, projectContext);
     command->setAttribute(Core::Command::CA_Hide);
     mproject->addAction(command, ProjectExplorer::Constants::G_PROJECT_BUILD);
-    connect(m_reparseQbsCtx, SIGNAL(triggered()), this, SLOT(reparseSelectedProject()));
+    connect(m_reparseQbsCtx, &QAction::triggered,
+            this, &QbsProjectManagerPlugin::reparseSelectedProject);
 
     m_buildFileCtx = new QAction(tr("Build"), this);
     command = Core::ActionManager::registerAction(m_buildFileCtx, Constants::ACTION_BUILD_FILE_CONTEXT, projectContext);
     command->setAttribute(Core::Command::CA_Hide);
     mfile->addAction(command, ProjectExplorer::Constants::G_FILE_OTHER);
-    connect(m_buildFileCtx, SIGNAL(triggered()), this, SLOT(buildFileContextMenu()));
+    connect(m_buildFileCtx, &QAction::triggered,
+            this, &QbsProjectManagerPlugin::buildFileContextMenu);
 
     m_buildFile = new Utils::ParameterAction(tr("Build File"), tr("Build File \"%1\""),
                                                    Utils::ParameterAction::AlwaysEnabled, this);
@@ -149,13 +152,14 @@ bool QbsProjectManagerPlugin::initialize(const QStringList &arguments, QString *
     command->setDescription(m_buildFile->text());
     command->setDefaultKeySequence(QKeySequence(tr("Ctrl+Alt+B")));
     mbuild->addAction(command, ProjectExplorer::Constants::G_BUILD_BUILD);
-    connect(m_buildFile, SIGNAL(triggered()), this, SLOT(buildFile()));
+    connect(m_buildFile, &QAction::triggered, this, &QbsProjectManagerPlugin::buildFile);
 
     m_buildProductCtx = new QAction(tr("Build"), this);
     command = Core::ActionManager::registerAction(m_buildProductCtx, Constants::ACTION_BUILD_PRODUCT_CONTEXT, projectContext);
     command->setAttribute(Core::Command::CA_Hide);
     msubproject->addAction(command, ProjectExplorer::Constants::G_PROJECT_BUILD);
-    connect(m_buildProductCtx, SIGNAL(triggered()), this, SLOT(buildProductContextMenu()));
+    connect(m_buildProductCtx, &QAction::triggered,
+            this, &QbsProjectManagerPlugin::buildProductContextMenu);
 
     m_buildProduct = new Utils::ParameterAction(tr("Build Product"), tr("Build Product \"%1\""),
                                                 Utils::ParameterAction::AlwaysEnabled, this);
@@ -165,13 +169,48 @@ bool QbsProjectManagerPlugin::initialize(const QStringList &arguments, QString *
     command->setDescription(m_buildFile->text());
     command->setDefaultKeySequence(QKeySequence(tr("Ctrl+Alt+Shift+B")));
     mbuild->addAction(command, ProjectExplorer::Constants::G_BUILD_BUILD);
-    connect(m_buildProduct, SIGNAL(triggered()), this, SLOT(buildProduct()));
+    connect(m_buildProduct, &QAction::triggered, this, &QbsProjectManagerPlugin::buildProduct);
+
+    m_cleanProductCtx = new QAction(tr("Clean"), this);
+    command = Core::ActionManager::registerAction(
+                m_cleanProductCtx, Constants::ACTION_CLEAN_PRODUCT_CONTEXT, projectContext);
+    command->setAttribute(Core::Command::CA_Hide);
+    msubproject->addAction(command, ProjectExplorer::Constants::G_PROJECT_BUILD);
+    connect(m_cleanProductCtx, &QAction::triggered,
+            this, &QbsProjectManagerPlugin::cleanProductContextMenu);
+
+    m_cleanProduct = new Utils::ParameterAction(tr("Clean Product"), tr("Clean Product \"%1\""),
+                                                Utils::ParameterAction::AlwaysEnabled, this);
+    command = Core::ActionManager::registerAction(m_cleanProduct, Constants::ACTION_CLEAN_PRODUCT);
+    command->setAttribute(Core::Command::CA_Hide);
+    command->setAttribute(Core::Command::CA_UpdateText);
+    mbuild->addAction(command, ProjectExplorer::Constants::G_BUILD_CLEAN);
+    connect(m_cleanProduct, &QAction::triggered, this, &QbsProjectManagerPlugin::cleanProduct);
+
+    m_rebuildProductCtx = new QAction(tr("Rebuild"), this);
+    command = Core::ActionManager::registerAction(
+                m_rebuildProductCtx, Constants::ACTION_REBUILD_PRODUCT_CONTEXT, projectContext);
+    command->setAttribute(Core::Command::CA_Hide);
+    msubproject->addAction(command, ProjectExplorer::Constants::G_PROJECT_BUILD);
+    connect(m_rebuildProductCtx, &QAction::triggered,
+            this, &QbsProjectManagerPlugin::rebuildProductContextMenu);
+
+    m_rebuildProduct = new Utils::ParameterAction(
+                tr("Rebuild Product"), tr("Rebuild Product \"%1\""),
+                Utils::ParameterAction::AlwaysEnabled, this);
+    command = Core::ActionManager::registerAction(m_rebuildProduct,
+                                                  Constants::ACTION_REBUILD_PRODUCT);
+    command->setAttribute(Core::Command::CA_Hide);
+    command->setAttribute(Core::Command::CA_UpdateText);
+    mbuild->addAction(command, ProjectExplorer::Constants::G_BUILD_REBUILD);
+    connect(m_rebuildProduct, &QAction::triggered, this, &QbsProjectManagerPlugin::rebuildProduct);
 
     m_buildSubprojectCtx = new QAction(tr("Build"), this);
     command = Core::ActionManager::registerAction(m_buildSubprojectCtx, Constants::ACTION_BUILD_SUBPROJECT_CONTEXT, projectContext);
     command->setAttribute(Core::Command::CA_Hide);
     msubproject->addAction(command, ProjectExplorer::Constants::G_PROJECT_BUILD);
-    connect(m_buildSubprojectCtx, SIGNAL(triggered()), this, SLOT(buildSubprojectContextMenu()));
+    connect(m_buildSubprojectCtx, &QAction::triggered,
+            this, &QbsProjectManagerPlugin::buildSubprojectContextMenu);
 
     m_buildSubproject = new Utils::ParameterAction(tr("Build Subproject"), tr("Build Subproject \"%1\""),
                                                 Utils::ParameterAction::AlwaysEnabled, this);
@@ -181,24 +220,64 @@ bool QbsProjectManagerPlugin::initialize(const QStringList &arguments, QString *
     command->setDescription(m_buildFile->text());
     command->setDefaultKeySequence(QKeySequence(tr("Ctrl+Shift+B")));
     mbuild->addAction(command, ProjectExplorer::Constants::G_BUILD_BUILD);
-    connect(m_buildSubproject, SIGNAL(triggered()), this, SLOT(buildSubproject()));
+    connect(m_buildSubproject, &QAction::triggered, this, &QbsProjectManagerPlugin::buildSubproject);
+
+    m_cleanSubprojectCtx = new QAction(tr("Clean"), this);
+    command = Core::ActionManager::registerAction(
+                m_cleanSubprojectCtx, Constants::ACTION_CLEAN_SUBPROJECT_CONTEXT, projectContext);
+    command->setAttribute(Core::Command::CA_Hide);
+    msubproject->addAction(command, ProjectExplorer::Constants::G_PROJECT_BUILD);
+    connect(m_cleanSubprojectCtx, &QAction::triggered,
+            this, &QbsProjectManagerPlugin::cleanSubprojectContextMenu);
+
+    m_cleanSubproject = new Utils::ParameterAction(
+                tr("Clean Subproject"), tr("Clean Subproject \"%1\""),
+                Utils::ParameterAction::AlwaysEnabled, this);
+    command = Core::ActionManager::registerAction(m_cleanSubproject,
+                                                  Constants::ACTION_CLEAN_SUBPROJECT);
+    command->setAttribute(Core::Command::CA_Hide);
+    command->setAttribute(Core::Command::CA_UpdateText);
+    mbuild->addAction(command, ProjectExplorer::Constants::G_BUILD_CLEAN);
+    connect(m_cleanSubproject, &QAction::triggered, this,
+            &QbsProjectManagerPlugin::cleanSubproject);
+
+    m_rebuildSubprojectCtx = new QAction(tr("Rebuild"), this);
+    command = Core::ActionManager::registerAction(
+                m_rebuildSubprojectCtx, Constants::ACTION_REBUILD_SUBPROJECT_CONTEXT,
+                projectContext);
+    command->setAttribute(Core::Command::CA_Hide);
+    msubproject->addAction(command, ProjectExplorer::Constants::G_PROJECT_BUILD);
+    connect(m_rebuildSubprojectCtx, &QAction::triggered,
+            this, &QbsProjectManagerPlugin::rebuildSubprojectContextMenu);
+
+    m_rebuildSubproject = new Utils::ParameterAction(
+                tr("Rebuild Subproject"), tr("Rebuild Subproject \"%1\""),
+                Utils::ParameterAction::AlwaysEnabled, this);
+    command = Core::ActionManager::registerAction(m_rebuildSubproject,
+                                                  Constants::ACTION_REBUILD_SUBPROJECT);
+    command->setAttribute(Core::Command::CA_Hide);
+    command->setAttribute(Core::Command::CA_UpdateText);
+    mbuild->addAction(command, ProjectExplorer::Constants::G_BUILD_REBUILD);
+    connect(m_rebuildSubproject, &QAction::triggered, this,
+            &QbsProjectManagerPlugin::rebuildSubproject);
+
 
     // Connect
     connect(ProjectTree::instance(), &ProjectTree::currentNodeChanged,
-            this, &QbsProjectManagerPlugin::nodeSelectionChanged);
+            this, &QbsProjectManagerPlugin::updateContextActions);
 
-    connect(BuildManager::instance(), SIGNAL(buildStateChanged(ProjectExplorer::Project*)),
-            this, SLOT(buildStateChanged(ProjectExplorer::Project*)));
+    connect(BuildManager::instance(), &BuildManager::buildStateChanged,
+            this, &QbsProjectManagerPlugin::projectChanged);
 
-    connect(Core::EditorManager::instance(), SIGNAL(currentEditorChanged(Core::IEditor*)),
-            this, SLOT(currentEditorChanged()));
+    connect(Core::EditorManager::instance(), &Core::EditorManager::currentEditorChanged,
+            this, &QbsProjectManagerPlugin::updateBuildActions);
 
-    connect(SessionManager::instance(), SIGNAL(projectAdded(ProjectExplorer::Project*)),
-            this, SLOT(projectWasAdded(ProjectExplorer::Project*)));
-    connect(SessionManager::instance(), SIGNAL(projectRemoved(ProjectExplorer::Project*)),
-            this, SLOT(projectWasRemoved()));
-    connect(SessionManager::instance(), SIGNAL(startupProjectChanged(ProjectExplorer::Project*)),
-            this, SLOT(currentProjectWasChanged(ProjectExplorer::Project*)));
+    connect(SessionManager::instance(), &SessionManager::projectAdded,
+            this, &QbsProjectManagerPlugin::projectWasAdded);
+    connect(SessionManager::instance(), &SessionManager::projectRemoved,
+            this, &QbsProjectManagerPlugin::updateBuildActions);
+    connect(SessionManager::instance(), &SessionManager::startupProjectChanged,
+            this, &QbsProjectManagerPlugin::updateReparseQbsAction);
 
     // Run initial setup routines
     updateContextActions();
@@ -218,55 +297,41 @@ void QbsProjectManagerPlugin::projectWasAdded(Project *project)
     if (!qbsProject)
         return;
 
-    connect(qbsProject, SIGNAL(projectParsingStarted()), this, SLOT(parsingStateChanged()));
-    connect(qbsProject, SIGNAL(projectParsingDone(bool)), this, SLOT(parsingStateChanged()));
-}
-
-void QbsProjectManagerPlugin::currentProjectWasChanged(Project *project)
-{
-    m_currentProject = qobject_cast<QbsProject *>(project);
-
-    updateReparseQbsAction();
-}
-
-void QbsProjectManagerPlugin::projectWasRemoved()
-{
-    m_editorNode = currentEditorNode();
-    m_editorProject = currentEditorProject();
-
-    updateBuildActions();
-}
-
-void QbsProjectManagerPlugin::nodeSelectionChanged(Node *node, Project *project)
-{
-    m_selectedNode = node;
-    m_selectedProject = qobject_cast<Internal::QbsProject *>(project);
-
-    updateContextActions();
+    connect(qbsProject, &Project::parsingStarted,
+            this, &QbsProjectManagerPlugin::projectChanged);
+    connect(qbsProject, &Project::parsingFinished,
+            this, &QbsProjectManagerPlugin::projectChanged);
 }
 
 void QbsProjectManagerPlugin::updateContextActions()
 {
-    bool isEnabled = !BuildManager::isBuilding(m_selectedProject)
-            && m_selectedProject && !m_selectedProject->isParsing()
-            && m_selectedNode && m_selectedNode->isEnabled();
+    QbsProject *project = qobject_cast<Internal::QbsProject *>(ProjectTree::currentProject());
+    const Node *node = ProjectTree::findCurrentNode();
+    bool isEnabled = !BuildManager::isBuilding(project)
+            && project && !project->isParsing()
+            && node && node->isEnabled();
 
-    bool isFile = m_selectedProject && m_selectedNode && (m_selectedNode->nodeType() == FileNodeType);
-    bool isProduct = m_selectedProject && m_selectedNode && dynamic_cast<QbsProductNode *>(m_selectedNode->projectNode());
-    QbsProjectNode *subproject = dynamic_cast<QbsProjectNode *>(m_selectedNode);
-    bool isSubproject = m_selectedProject && subproject && subproject != m_selectedProject->rootProjectNode();
+    bool isFile = project && node && (node->nodeType() == NodeType::File);
+    const bool isProduct = project && node && dynamic_cast<const QbsProductNode *>(node);
+    const QbsProjectNode *subproject = dynamic_cast<const QbsProjectNode *>(node);
+    bool isSubproject = project && subproject && subproject != project->rootProjectNode();
 
     m_reparseQbsCtx->setEnabled(isEnabled);
     m_buildFileCtx->setEnabled(isEnabled && isFile);
     m_buildProductCtx->setVisible(isEnabled && isProduct);
+    m_cleanProductCtx->setVisible(isEnabled && isProduct);
+    m_rebuildProductCtx->setVisible(isEnabled && isProduct);
     m_buildSubprojectCtx->setVisible(isEnabled && isSubproject);
+    m_cleanSubprojectCtx->setVisible(isEnabled && isSubproject);
+    m_rebuildSubprojectCtx->setVisible(isEnabled && isSubproject);
 }
 
 void QbsProjectManagerPlugin::updateReparseQbsAction()
 {
-    m_reparseQbs->setEnabled(m_currentProject
-                             && !BuildManager::isBuilding(m_currentProject)
-                             && !m_currentProject->isParsing());
+    QbsProject *project = qobject_cast<QbsProject *>(SessionManager::startupProject());
+    m_reparseQbs->setEnabled(project
+                             && !BuildManager::isBuilding(project)
+                             && !project->isParsing());
 }
 
 void QbsProjectManagerPlugin::updateBuildActions()
@@ -280,23 +345,24 @@ void QbsProjectManagerPlugin::updateBuildActions()
     QString productName;
     QString subprojectName;
 
-    if (m_editorNode) {
-        enabled = m_editorProject
-                && !BuildManager::isBuilding(m_editorProject)
-                && !m_editorProject->isParsing();
+    if (Node *editorNode = currentEditorNode()) {
+        QbsProject *editorProject = currentEditorProject();
+        enabled = editorProject
+                && !BuildManager::isBuilding(editorProject)
+                && !editorProject->isParsing();
 
-        fileName = m_editorNode->filePath().fileName();
-        fileVisible = m_editorProject && m_editorNode && dynamic_cast<QbsBaseProjectNode *>(m_editorNode->projectNode());
+        fileName = editorNode->filePath().fileName();
+        fileVisible = editorProject && editorNode && dynamic_cast<QbsBaseProjectNode *>(editorNode->parentProjectNode());
 
-        QbsProductNode *productNode
-                = dynamic_cast<QbsProductNode *>(m_editorNode ? m_editorNode->projectNode() : 0);
+        QbsProductNode *productNode =
+            dynamic_cast<QbsProductNode *>(editorNode ? editorNode->parentProjectNode() : 0);
         if (productNode) {
             productVisible = true;
             productName = productNode->displayName();
         }
-        QbsProjectNode *subprojectNode
-                = dynamic_cast<QbsProjectNode *>(productNode ? productNode->parentFolderNode() : 0);
-        if (subprojectNode && m_editorProject && subprojectNode != m_editorProject->rootProjectNode()) {
+        QbsProjectNode *subprojectNode =
+            dynamic_cast<QbsProjectNode *>(productNode ? productNode->parentFolderNode() : 0);
+        if (subprojectNode && editorProject && subprojectNode != editorProject->rootProjectNode()) {
             subprojectVisible = true;
             subprojectName = subprojectNode->displayName();
         }
@@ -309,115 +375,185 @@ void QbsProjectManagerPlugin::updateBuildActions()
     m_buildProduct->setEnabled(enabled);
     m_buildProduct->setVisible(productVisible);
     m_buildProduct->setParameter(productName);
+    m_cleanProduct->setEnabled(enabled);
+    m_cleanProduct->setVisible(productVisible);
+    m_cleanProduct->setParameter(productName);
+    m_rebuildProduct->setEnabled(enabled);
+    m_rebuildProduct->setVisible(productVisible);
+    m_rebuildProduct->setParameter(productName);
 
     m_buildSubproject->setEnabled(enabled);
     m_buildSubproject->setVisible(subprojectVisible);
     m_buildSubproject->setParameter(subprojectName);
+    m_cleanSubproject->setEnabled(enabled);
+    m_cleanSubproject->setVisible(subprojectVisible);
+    m_cleanSubproject->setParameter(subprojectName);
+    m_rebuildSubproject->setEnabled(enabled);
+    m_rebuildSubproject->setVisible(subprojectVisible);
+    m_rebuildSubproject->setParameter(subprojectName);
 }
 
-void QbsProjectManagerPlugin::buildStateChanged(Project *project)
-{
-    if (project == m_currentProject)
-        updateReparseQbsAction();
-
-    if (project == m_selectedProject)
-        updateContextActions();
-
-    m_editorNode = currentEditorNode();
-    m_editorProject = currentEditorProject();
-    if (project == m_editorProject)
-        updateBuildActions();
-}
-
-void QbsProjectManagerPlugin::parsingStateChanged()
+void QbsProjectManagerPlugin::projectChanged()
 {
     QbsProject *project = qobject_cast<QbsProject *>(sender());
 
-    if (!project || project == m_currentProject)
+    if (!project || project == SessionManager::startupProject())
         updateReparseQbsAction();
 
-    if (!project || project == m_selectedProject)
+    if (!project || project == ProjectTree::currentProject())
         updateContextActions();
 
-    m_editorNode = currentEditorNode();
-    m_editorProject = currentEditorProject();
-    if (!project || project == m_editorProject)
+    if (!project || project == currentEditorProject())
         updateBuildActions();
 }
 
-void QbsProjectManagerPlugin::currentEditorChanged()
-{
-    m_editorNode = currentEditorNode();
-    m_editorProject = currentEditorProject();
-
-    updateBuildActions();
-}
-
 void QbsProjectManagerPlugin::buildFileContextMenu()
 {
-    QTC_ASSERT(m_selectedNode, return);
-    QTC_ASSERT(m_selectedProject, return);
-
-    buildSingleFile(m_selectedProject, m_selectedNode->filePath().toString());
+    const Node *node = ProjectTree::findCurrentNode();
+    QTC_ASSERT(node, return);
+    QbsProject *project = dynamic_cast<QbsProject *>(ProjectTree::currentProject());
+    QTC_ASSERT(project, return);
+    buildSingleFile(project, node->filePath().toString());
 }
 
 void QbsProjectManagerPlugin::buildFile()
 {
-    if (!m_editorProject || !m_editorNode)
+    Node *node = currentEditorNode();
+    QbsProject *project = currentEditorProject();
+    if (!project || !node)
         return;
 
-    buildSingleFile(m_editorProject, m_editorNode->filePath().toString());
+    buildSingleFile(project, node->filePath().toString());
 }
 
 void QbsProjectManagerPlugin::buildProductContextMenu()
 {
-    QTC_ASSERT(m_selectedNode, return);
-    QTC_ASSERT(m_selectedProject, return);
+    runStepsForProductContextMenu({Core::Id(ProjectExplorer::Constants::BUILDSTEPS_BUILD)});
+}
 
-    const QbsProductNode * const productNode = dynamic_cast<QbsProductNode *>(m_selectedNode);
+void QbsProjectManagerPlugin::cleanProductContextMenu()
+{
+    runStepsForProductContextMenu({Core::Id(ProjectExplorer::Constants::BUILDSTEPS_CLEAN)});
+}
+
+void QbsProjectManagerPlugin::rebuildProductContextMenu()
+{
+    runStepsForProductContextMenu({
+        Core::Id(ProjectExplorer::Constants::BUILDSTEPS_CLEAN),
+        Core::Id(ProjectExplorer::Constants::BUILDSTEPS_BUILD)
+    });
+}
+
+void QbsProjectManagerPlugin::runStepsForProductContextMenu(const QList<Core::Id> &stepTypes)
+{
+    const Node *node = ProjectTree::findCurrentNode();
+    QTC_ASSERT(node, return);
+    QbsProject *project = dynamic_cast<QbsProject *>(ProjectTree::currentProject());
+    QTC_ASSERT(project, return);
+
+    const QbsProductNode * const productNode = dynamic_cast<const QbsProductNode *>(node);
     QTC_ASSERT(productNode, return);
 
-    buildProducts(m_selectedProject,
-                  QStringList(QbsProject::uniqueProductName(productNode->qbsProductData())));
+    runStepsForProducts(project, {QbsProject::uniqueProductName(productNode->qbsProductData())},
+                        {stepTypes});
 }
 
 void QbsProjectManagerPlugin::buildProduct()
 {
-    if (!m_editorProject || !m_editorNode)
+    runStepsForProduct({Core::Id(ProjectExplorer::Constants::BUILDSTEPS_BUILD)});
+}
+
+void QbsProjectManagerPlugin::cleanProduct()
+{
+    runStepsForProduct({Core::Id(ProjectExplorer::Constants::BUILDSTEPS_CLEAN)});
+}
+
+void QbsProjectManagerPlugin::rebuildProduct()
+{
+    runStepsForProduct({
+        Core::Id(ProjectExplorer::Constants::BUILDSTEPS_CLEAN),
+        Core::Id(ProjectExplorer::Constants::BUILDSTEPS_BUILD),
+    });
+}
+
+void QbsProjectManagerPlugin::runStepsForProduct(const QList<Core::Id> &stepTypes)
+{
+    Node *node = currentEditorNode();
+    if (!node)
         return;
-
-    QbsProductNode *product = dynamic_cast<QbsProductNode *>(m_editorNode->projectNode());
-
+    QbsProductNode *product = dynamic_cast<QbsProductNode *>(node->parentProjectNode());
     if (!product)
         return;
-
-    buildProducts(m_editorProject,
-                  QStringList(QbsProject::uniqueProductName(product->qbsProductData())));
+    QbsProject *project = currentEditorProject();
+    if (!project)
+        return;
+    runStepsForProducts(project, {QbsProject::uniqueProductName(product->qbsProductData())},
+                        {stepTypes});
 }
 
 void QbsProjectManagerPlugin::buildSubprojectContextMenu()
 {
-    QTC_ASSERT(m_selectedNode, return);
-    QTC_ASSERT(m_selectedProject, return);
+    runStepsForSubprojectContextMenu({Core::Id(ProjectExplorer::Constants::BUILDSTEPS_BUILD)});
+}
 
-    QbsProjectNode *subProject = dynamic_cast<QbsProjectNode *>(m_selectedNode);
+void QbsProjectManagerPlugin::cleanSubprojectContextMenu()
+{
+    runStepsForSubprojectContextMenu({Core::Id(ProjectExplorer::Constants::BUILDSTEPS_CLEAN)});
+}
+
+void QbsProjectManagerPlugin::rebuildSubprojectContextMenu()
+{
+    runStepsForSubprojectContextMenu({
+        Core::Id(ProjectExplorer::Constants::BUILDSTEPS_CLEAN),
+        Core::Id(ProjectExplorer::Constants::BUILDSTEPS_BUILD)
+    });
+}
+
+void QbsProjectManagerPlugin::runStepsForSubprojectContextMenu(const QList<Core::Id> &stepTypes)
+{
+    const Node *node = ProjectTree::findCurrentNode();
+    QTC_ASSERT(node, return);
+    QbsProject *project = dynamic_cast<QbsProject *>(ProjectTree::currentProject());
+    QTC_ASSERT(project, return);
+
+    const QbsProjectNode *subProject = dynamic_cast<const QbsProjectNode *>(node);
     QTC_ASSERT(subProject, return);
 
     QStringList toBuild;
     foreach (const qbs::ProductData &data, subProject->qbsProjectData().allProducts())
         toBuild << QbsProject::uniqueProductName(data);
 
-    buildProducts(m_selectedProject, toBuild);
+    runStepsForProducts(project, toBuild, {stepTypes});
 }
 
 void QbsProjectManagerPlugin::buildSubproject()
 {
-    if (!m_editorNode || !m_editorProject)
+    runStepsForSubproject({Core::Id(ProjectExplorer::Constants::BUILDSTEPS_BUILD)});
+}
+
+void QbsProjectManagerPlugin::cleanSubproject()
+{
+    runStepsForSubproject({Core::Id(ProjectExplorer::Constants::BUILDSTEPS_CLEAN)});
+}
+
+void QbsProjectManagerPlugin::rebuildSubproject()
+{
+    runStepsForSubproject({
+        Core::Id(ProjectExplorer::Constants::BUILDSTEPS_CLEAN),
+        Core::Id(ProjectExplorer::Constants::BUILDSTEPS_BUILD)
+    });
+}
+
+void QbsProjectManagerPlugin::runStepsForSubproject(const QList<Core::Id> &stepTypes)
+{
+    Node *editorNode = currentEditorNode();
+    QbsProject *editorProject = currentEditorProject();
+    if (!editorNode || !editorProject)
         return;
 
     QbsProjectNode *subproject = 0;
-    QbsBaseProjectNode *start = dynamic_cast<QbsBaseProjectNode *>(m_editorNode->projectNode());
-    while (start && start != m_editorProject->rootProjectNode()) {
+    QbsBaseProjectNode *start = dynamic_cast<QbsBaseProjectNode *>(editorNode->parentProjectNode());
+    while (start && start != editorProject->rootProjectNode()) {
         QbsProjectNode *tmp = dynamic_cast<QbsProjectNode *>(start);
         if (tmp) {
             subproject = tmp;
@@ -433,7 +569,7 @@ void QbsProjectManagerPlugin::buildSubproject()
     foreach (const qbs::ProductData &data, subproject->qbsProjectData().allProducts())
         toBuild << QbsProject::uniqueProductName(data);
 
-    buildProducts(m_editorProject, toBuild);
+    runStepsForProducts(editorProject, toBuild, {stepTypes});
 }
 
 void QbsProjectManagerPlugin::buildFiles(QbsProject *project, const QStringList &files,
@@ -458,8 +594,7 @@ void QbsProjectManagerPlugin::buildFiles(QbsProject *project, const QStringList
 
     const Core::Id buildStep = ProjectExplorer::Constants::BUILDSTEPS_BUILD;
 
-    const QString name = ProjectExplorerPlugin::displayNameForStepId(buildStep);
-    BuildManager::buildList(bc->stepList(buildStep), name);
+    BuildManager::buildList(bc->stepList(buildStep));
 
     bc->setChangedFiles(QStringList());
     bc->setActiveFileTags(QStringList());
@@ -467,11 +602,11 @@ void QbsProjectManagerPlugin::buildFiles(QbsProject *project, const QStringList
 
 void QbsProjectManagerPlugin::buildSingleFile(QbsProject *project, const QString &file)
 {
-    buildFiles(project, QStringList(file), QStringList()
-               << QLatin1String("obj") << QLatin1String("hpp"));
+    buildFiles(project, QStringList(file), QStringList({"obj", "hpp"}));
 }
 
-void QbsProjectManagerPlugin::buildProducts(QbsProject *project, const QStringList &products)
+void QbsProjectManagerPlugin::runStepsForProducts(QbsProject *project,
+        const QStringList &products, const QList<Core::Id> &stepTypes)
 {
     QTC_ASSERT(project, return);
     QTC_ASSERT(!products.isEmpty(), return);
@@ -483,28 +618,31 @@ void QbsProjectManagerPlugin::buildProducts(QbsProject *project, const QStringLi
     if (!bc)
         return;
 
-    if (!ProjectExplorerPlugin::saveModifiedFiles())
+    if (stepTypes.contains(ProjectExplorer::Constants::BUILDSTEPS_BUILD)
+            && !ProjectExplorerPlugin::saveModifiedFiles()) {
         return;
+    }
 
     bc->setChangedFiles(QStringList());
     bc->setProducts(products);
-
-    const Core::Id buildStep = ProjectExplorer::Constants::BUILDSTEPS_BUILD;
-
-    const QString name = ProjectExplorerPlugin::displayNameForStepId(buildStep);
-    BuildManager::buildList(bc->stepList(buildStep), name);
-
+    QList<ProjectExplorer::BuildStepList *> stepLists;
+    QStringList stepListNames;
+    for (const Core::Id &stepType : stepTypes) {
+        stepLists << bc->stepList(stepType);
+        stepListNames <<ProjectExplorerPlugin::displayNameForStepId(stepType);
+    }
+    BuildManager::buildLists(stepLists, stepListNames);
     bc->setProducts(QStringList());
 }
 
 void QbsProjectManagerPlugin::reparseSelectedProject()
 {
-    reparseProject(m_selectedProject);
+    reparseProject(dynamic_cast<QbsProject *>(ProjectTree::currentProject()));
 }
 
 void QbsProjectManagerPlugin::reparseCurrentProject()
 {
-    reparseProject(m_currentProject);
+    reparseProject(dynamic_cast<QbsProject *>(SessionManager::startupProject()));
 }
 
 void QbsProjectManagerPlugin::reparseProject(QbsProject *project)
