@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,27 +9,24 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
+#pragma once
 
-#ifndef FORMEDITORWIDGET_H
-#define FORMEDITORWIDGET_H
+#include <documentwarningwidget.h>
+
+#include <coreplugin/icontext.h>
 
 #include <QWidget>
 #include <QPointer>
@@ -42,6 +39,7 @@ namespace QmlDesigner {
 
 class ZoomAction;
 class LineEditAction;
+class BackgroundAction;
 class FormEditorView;
 class FormEditorScene;
 class FormEditorGraphicsView;
@@ -57,7 +55,6 @@ public:
 
     ZoomAction *zoomAction() const;
     QAction *showBoundingRectAction() const;
-    QAction *selectOnlyContentItemsAction() const;
     QAction *snappingAction() const;
     QAction *snappingAndAnchoringAction() const;
 
@@ -67,7 +64,7 @@ public:
     double spacing() const;
     double containerPadding() const;
 
-    QString contextHelpId() const;
+    void contextHelpId(const Core::IContext::HelpIdCallback &callback) const;
 
     void setRootItemRect(const QRectF &rect);
     QRectF rootItemRect() const;
@@ -79,15 +76,26 @@ public:
 
     void setFocus();
 
+    void showErrorMessageBox(const QList<DocumentMessage> &errors);
+    void hideErrorMessageBox();
+
+    void showWarningMessageBox(const QList<DocumentMessage> &warnings);
+
+    void exportAsImage(const QRectF &boundingRect);
+
+    FormEditorGraphicsView *graphicsView() const;
+
 protected:
     void wheelEvent(QWheelEvent *event);
     QActionGroup *toolActionGroup() const;
+    DocumentWarningWidget *errorWidget();
 
-private slots:
+private:
     void changeTransformTool(bool checked);
     void setZoomLevel(double zoomLevel);
     void changeRootItemWidth(const QString &widthText);
     void changeRootItemHeight(const QString &heightText);
+    void changeBackgound(const QColor &color);
     void resetNodeInstanceView();
 
 private:
@@ -101,13 +109,11 @@ private:
     QPointer<QAction> m_snappingAndAnchoringAction;
     QPointer<QAction> m_noSnappingAction;
     QPointer<QAction> m_showBoundingRectAction;
-    QPointer<QAction> m_selectOnlyContentItemsAction;
     QPointer<LineEditAction> m_rootWidthAction;
     QPointer<LineEditAction> m_rootHeightAction;
+    QPointer<BackgroundAction> m_backgroundAction;
     QPointer<QAction> m_resetAction;
+    QPointer<DocumentWarningWidget> m_documentErrorWidget;
 };
 
-
-}
-#endif //FORMEDITORWIDGET_H
-
+} // namespace QmlDesigner
